import os
import unittest

from hovercraft.generate import rst2html
from hovercraft.template import Template
from .test_data import HTML_OUTPUTS

TEST_DATA = os.path.join(os.path.split(__file__)[0], 'test_data')


class GeneratorTests(unittest.TestCase):
    """Tests that the resulting HTML is correct.

    This tests the whole path except the copying of files."""

    def test_small(self):
        template = Template(os.path.join(TEST_DATA, 'minimal'))
        html, deps = rst2html(os.path.join(TEST_DATA, 'simple.rst'), template)
        self.assertEqual(html, HTML_OUTPUTS['simple'])

    def test_big(self):
        template = Template(os.path.join(TEST_DATA, 'maximal'))
        html, deps = rst2html(os.path.join(TEST_DATA, 'advanced.rst'), template)
        self.assertEqual(html, HTML_OUTPUTS['advanced'])

    def test_presenter_notes(self):
        template = Template(os.path.join(TEST_DATA, 'maximal'))
        html, deps = rst2html(os.path.join(TEST_DATA, 'presenter-notes.rst'), template)
        self.assertEqual(html, HTML_OUTPUTS['presenter-notes'])

    def test_skip_presenter_notes(self):
        template = Template(os.path.join(TEST_DATA, 'maximal'))
        html, deps = rst2html(os.path.join(TEST_DATA, 'presenter-notes.rst'), template, skip_notes=True)
        self.assertEqual(html, HTML_OUTPUTS['skip-presenter-notes'])

    def test_comments(self):
        template = Template(os.path.join(TEST_DATA, 'minimal'))
        html, deps = rst2html(os.path.join(TEST_DATA, 'comment.rst'), template)
        self.assertEqual(html, HTML_OUTPUTS['comment'])

    def test_slide_with_class(self):
        template = Template(os.path.join(TEST_DATA, 'minimal'))
        html, deps = rst2html(os.path.join(TEST_DATA, 'slide_class.rst'), template)
        self.assertEqual(html, HTML_OUTPUTS['slide_with_class'])

    def test_table(self):
        template = Template(os.path.join(TEST_DATA, 'minimal'))
        html, deps = rst2html(os.path.join(TEST_DATA, 'table.rst'), template)
        self.assertEqual(html, HTML_OUTPUTS['table'])

    def test_class_directive(self):
        template = Template(os.path.join(TEST_DATA, 'minimal'))
        html, deps = rst2html(os.path.join(TEST_DATA, 'class.rst'), template)
        self.assertEqual(html, HTML_OUTPUTS['class_directive'])

    def test_container_directive(self):
        template = Template(os.path.join(TEST_DATA, 'minimal'))
        html, deps = rst2html(os.path.join(TEST_DATA, 'container.rst'), template)
        self.assertEqual(html, HTML_OUTPUTS['container_directive'])

    def test_include(self):
        template = Template(os.path.join(TEST_DATA, 'minimal'))
        html, deps = rst2html(os.path.join(TEST_DATA, 'include.rst'), template)
        self.assertIn(b'Presentation with an include</h1>', html)
        # Make sure the simple presentation was included:
        self.assertIn(b'Simple Presentation</h1>', html)

if __name__ == '__main__':
    unittest.main()



import os
import sys
from tempfile import TemporaryDirectory
import unittest

from hovercraft import main
from .test_data import HTML_OUTPUTS

TEST_DATA = os.path.join(os.path.split(__file__)[0], 'test_data')


class HTMLTests(unittest.TestCase):
    """Test the procedure from rst to html"""

    def test_small(self):
        with TemporaryDirectory() as tmpdir:
            sys.argv = [
                'bin/hovercraft',
                '-t' + os.path.join(TEST_DATA, 'minimal'),
                os.path.join(TEST_DATA, 'simple.rst'),
                tmpdir,
            ]

            main()

            with open(os.path.join(tmpdir, 'index.html'), 'rb') as outfile:
                self.assertEqual(outfile.read(), HTML_OUTPUTS['simple'])

            js_files = os.listdir(os.path.join(tmpdir, 'js'))
            self.assertEqual(set(js_files), {'impress.js', 'hovercraft-minimal.js'})

    def test_extra_css(self):
        with TemporaryDirectory() as tmpdir:
            sys.argv = [
                'bin/hovercraft',
                '-t' + os.path.join(TEST_DATA, 'maximal'),
                '-c' + os.path.join(TEST_DATA, 'extra.css'),
                '-n',
                os.path.join(TEST_DATA, 'simple.rst'),
                tmpdir,
            ]

            main()

            with open(os.path.join(tmpdir, 'index.html'), 'rb') as outfile:
                self.assertEqual(outfile.read(), HTML_OUTPUTS['extra_css'])

            out_files = os.listdir(tmpdir)
            self.assertEqual(set(out_files),
                             {'extra.css', 'index.html', 'js', 'css', 'images', 'fonts', 'directory'})

    def test_extra_js(self):
        with TemporaryDirectory() as tmpdir:
            sys.argv = [
                'bin/hovercraft',
                '-t' + os.path.join(TEST_DATA, 'maximal'),
                '-j' + os.path.join(TEST_DATA, 'extra.js'),
                '-n',
                os.path.join(TEST_DATA, 'simple.rst'),
                tmpdir,
            ]

            main()

            with open(os.path.join(tmpdir, 'index.html'), 'rb') as outfile:
                self.assertEqual(outfile.read(), HTML_OUTPUTS['extra_js'])

            out_files = os.listdir(tmpdir)
            self.assertEqual(set(out_files),
                             {'extra.js', 'index.html', 'js', 'css', 'images', 'fonts', 'directory'})

    def test_big(self):
        with TemporaryDirectory() as tmpdir:
            sys.argv = [
                'bin/hovercraft',
                '-t' + os.path.join(TEST_DATA, 'maximal', 'template.cfg'),
                os.path.join(TEST_DATA, 'advanced.rst'),
                tmpdir,
            ]

            main()

            with open(os.path.join(tmpdir, 'index.html'), 'rb') as outfile:
                # We have verified the contents in test_generator.py, let's
                # just check that it writes the right thing:
                self.assertEqual(outfile.read(), HTML_OUTPUTS['advanced'])

            out_files = os.listdir(tmpdir)
            self.assertEqual(set(out_files),
                             {'extra.css', 'extra.js', 'index.html', 'js', 'css', 'images', 'fonts', 'directory'})

            # Make sure the whole tree of the directory resource was copies,
            # except .dont-include, which should not be included.
            out_files = os.listdir(os.path.join(tmpdir, 'directory'))
            self.assertEqual(set(out_files),
                             {'subdir', 'hovercraft_logo.png', 'print.css'})
            out_files = os.listdir(os.path.join(tmpdir, 'directory', 'subdir'))
            self.assertEqual(set(out_files),
                             {'afile'})

            js_files = os.listdir(os.path.join(tmpdir, 'js'))
            self.assertEqual(set(js_files),
                             {'impress.js', 'hovercraft.js', 'impressConsole.js', 'dummy.js'})
            css_files = os.listdir(os.path.join(tmpdir, 'css'))
            self.assertEqual(set(css_files), {'print.css', 'style.css', 'impressConsole.css'})
            image_files = os.listdir(os.path.join(tmpdir, 'images'))
            self.assertEqual(set(image_files), {'hovercraft_logo.png'})
            font_files = os.listdir(os.path.join(tmpdir, 'fonts'))
            self.assertEqual(set(font_files), {
                'texgyreschola-regular-webfont.ttf',
                'texgyreschola-regular-webfont.eot',
                'texgyreschola-regular-webfont.woff',
                'texgyreschola-regular-webfont.svg',
            })

    def test_skip_notes(self):
        with TemporaryDirectory() as tmpdir:
            sys.argv = [
                'bin/hovercraft',
                '-t' + os.path.join(TEST_DATA, 'maximal'),
                '-n',
                os.path.join(TEST_DATA, 'presenter-notes.rst'),
                tmpdir,
            ]

            main()

            with open(os.path.join(tmpdir, 'index.html'), 'rb') as outfile:
                # We have verified the contents in test_generator.py, let's
                # just check that it writes the right thing:
                self.assertEqual(outfile.read(), HTML_OUTPUTS['skip-presenter-notes'])

            out_files = os.listdir(tmpdir)
            self.assertEqual(set(out_files), {'index.html', 'js', 'css', 'images', 'fonts', 'directory'})
            js_files = os.listdir(os.path.join(tmpdir, 'js'))
            self.assertEqual(set(js_files),
                             {'impress.js', 'hovercraft.js', 'impressConsole.js', 'dummy.js'})
            css_files = os.listdir(os.path.join(tmpdir, 'css'))
            self.assertEqual(set(css_files), {'print.css', 'style.css', 'impressConsole.css'})
            image_files = os.listdir(os.path.join(tmpdir, 'images'))
            self.assertEqual(set(image_files), {'hovercraft_logo.png'})
            font_files = os.listdir(os.path.join(tmpdir, 'fonts'))
            self.assertEqual(set(font_files), {
                'texgyreschola-regular-webfont.ttf',
                'texgyreschola-regular-webfont.eot',
                'texgyreschola-regular-webfont.woff',
                'texgyreschola-regular-webfont.svg',
            })

    def test_default_template(self):
        with TemporaryDirectory() as tmpdir:
            # Adding a non-existant subdir, to test that it gets created.
            tmpdir = os.path.join(tmpdir, 'foo')

            sys.argv = [
                'bin/hovercraft',
                os.path.join(TEST_DATA, 'advanced.rst'),
                tmpdir,
            ]
            main()

            with open(os.path.join(tmpdir, 'index.html'), 'rb') as outfile:
                self.assertEqual(outfile.read(), HTML_OUTPUTS['default-template'])

            js_files = os.listdir(os.path.join(tmpdir, 'js'))
            self.assertEqual(set(js_files), {'impress.js', 'hovercraft.js',
                                             'impressConsole.js'})
            css_files = os.listdir(os.path.join(tmpdir, 'css'))
            self.assertEqual(set(css_files), {'hovercraft.css',
                                              'impressConsole.css', 'highlight.css'})
            image_files = os.listdir(os.path.join(tmpdir, 'images'))
            self.assertEqual(set(image_files), {'hovercraft_logo.png'})

    def test_auto_console(self):
        with TemporaryDirectory() as tmpdir:
            # Adding a non-existant subdir, to test that it gets created.
            tmpdir = os.path.join(tmpdir, 'foo')

            sys.argv = [
                'bin/hovercraft',
                '-a',
                os.path.join(TEST_DATA, 'simple.rst'),
                tmpdir,
            ]
            main()

            with open(os.path.join(tmpdir, 'index.html'), 'rb') as outfile:
                result = outfile.read()
                self.assertIn(b'auto-console="True"', result)

    def test_subdirectory_css(self):
        with TemporaryDirectory() as tmpdir:
            sys.argv = [
                'bin/hovercraft',
                os.path.join(TEST_DATA, 'subdir-css.rst'),
                tmpdir,
            ]

            main()

            out_files = os.listdir(tmpdir)
            self.assertEqual(set(out_files), {'index.html', 'js', 'css', 'images'})
            css_files = os.listdir(os.path.join(tmpdir, 'css'))
            self.assertEqual(set(css_files),
                             {'hovercraft.css', 'highlight.css', 'sub.css', 'sub2.css', 'impressConsole.css'})
            image_files = os.listdir(os.path.join(tmpdir, 'images'))
            self.assertEqual(set(image_files), {'hovercraft_logo.png'})

    def test_mathjax(self):
        with TemporaryDirectory() as tmpdir:
            sys.argv = [
                'bin/hovercraft',
                os.path.join(TEST_DATA, 'math.rst'),
                tmpdir,
            ]

            main()

            with open(os.path.join(tmpdir, 'index.html'), 'rb') as outfile:
                result = outfile.read()
                self.assertIn(b'<script type="text/javascript" src="https://cdnjs.cloudflare.com/ajax/libs/mathjax/2.7.1/MathJax.js?config=TeX-MML-AM_CHTML">', result)
                self.assertIn(br'<div class="math-block ">$$\begin{align}dS = \frac{dQ}{T}\end{align}$$</div>', result)
                self.assertIn(br'<span class="math ">\(S = k \log W\)</span>', result)

if __name__ == '__main__':
    unittest.main()



import math

from svg.path import parse_path

DEFAULT_MOVEMENT = 1600  # If no other movement is specified, go 1600px to the right.
POSITION_ATTRIBS = ['data-x', 'data-y', 'data-z', 'data-rotate-x',
                    'data-rotate-y', 'data-rotate-z', 'data-scale']


def gather_positions(tree):
    """Makes a list of positions and position commands from the tree"""
    pos = {'data-x': 'r0',
           'data-y': 'r0',
           'data-z': 'r0',
           'data-rotate-x': 'r0',
           'data-rotate-y': 'r0',
           'data-rotate-z': 'r0',
           'data-scale': 'r0',
           'is_path': False
           }

    steps = 0
    default_movement = True

    for step in tree.findall('step'):
        steps += 1

        for key in POSITION_ATTRIBS:
            value = step.get(key)

            if value is not None:
                # We have a new value
                default_movement = False  # No longer use the default movement
                pos[key] = value
            elif pos[key] and not pos[key].startswith('r'):
                # The old value was absolute and no new value, so stop
                pos[key] = 'r0'
            # We had no new value, and the old value was a relative
            # movement, so we just keep moving.

        if steps == 1 and pos['data-scale'] == 'r0':
            # No scale given for first slide, it needs to start at 1
            pos['data-scale'] = '1'

        if default_movement and steps != 1:
            # No positioning has been given, use default:
            pos['data-x'] = 'r%s' % DEFAULT_MOVEMENT

        if 'data-rotate' in step.attrib:
            # data-rotate is an alias for data-rotate-z
            pos['data-rotate-z'] = step.get('data-rotate')
            del step.attrib['data-rotate']

        if 'hovercraft-path' in step.attrib:
            # Path given x and y will be calculated from the path
            default_movement = False  # No longer use the default movement
            pos['is_path'] = True
            # Add the path spec
            pos['path'] = step.attrib['hovercraft-path']
            yield pos.copy()
            # And get rid of it for the next step
            del pos['path']
        else:
            if 'data-x' in step.attrib or 'data-y' in step.attrib:
                # No longer using a path
                pos['is_path'] = False
            yield pos.copy()


def _coord_to_pos(coord):
    return {'data-x': int(coord.real), 'data-y': int(coord.imag)}


def _pos_to_cord(coord):
    return coord['data-x'] + coord['data-y'] * 1j


def _path_angle(path, point):
    start = point - 0.01
    end = point + 0.01
    if start < 0:
        start = 0
        end += 0.01
    elif end > 1:
        end = 1
        start -= 0.01

    distance = path.point(end) - path.point(start)
    hyp = math.hypot(distance.real, distance.imag)
    result = math.degrees(math.asin(distance.imag / hyp))

    if distance.real < 0:
        result = -180 - result

    if abs(result) < 0.1:
        result = 0

    return result


def num(s):
    try:
        return int(s)
    except ValueError:
        return float(s)


def _update_position(pos1, pos2):

    for key in POSITION_ATTRIBS:
        val = pos2.get(key)
        if val is not None:
            plus = val.find("+")
            minus = val.find("-")
            if plus > -1:
                newval = num(val[plus+1:])
                pos1[key + "-rel"] = val[0:plus]
            elif minus > -1 and not val.startswith("r-"):
                newval = num(val[minus:])
                pos1[key + "-rel"] = val[0:minus]
            else:
                if val[0] == 'r':
                # Relative movement
                    newval = pos1[key] + num(val[1:])
                else:
                    newval = num(val)
                    pos1.pop(key+"-rel",  None)
            pos1[key] = newval


def calculate_positions(positions):
    """Calculates position information"""
    current_position = {'data-x': 0,
                        'data-y': 0,
                        'data-z': 0,
                        'data-rotate-x': 0,
                        'data-rotate-y': 0,
                        'data-rotate-z': 0,
                        'data-scale': 1,
                        }

    positer = iter(positions)
    position = next(positer)
    _update_position(current_position, position)

    while True:

        if 'path' in position:
            # Start of a new path!
            path = position['path']
            # Follow the path specification
            first_point = _pos_to_cord(current_position)

            # Paths that end in Z or z are closed.
            closed_path = path.strip()[-1].upper() == 'Z'
            path = parse_path(path)

            # Find out how many positions should be calculated:
            count = 1
            last = False
            deferred_positions = []
            while True:
                try:
                    position = next(positer)
                    deferred_positions.append(position)
                except StopIteration:
                    last = True  # This path goes to the end
                    break
                if not position.get('is_path') or 'path' in position:
                    # The end of the path, or the start of a new one
                    break
                count += 1

            if count < 2:
                raise AssertionError("The path specification is only used for "
                                     "one slide, which makes it pointless.")

            if closed_path:
                # This path closes in on itself. Skip the last part, so that
                # the first and last step doesn't overlap.
                endcount = count + 1
            else:
                endcount = count

            multiplier = (endcount * DEFAULT_MOVEMENT) / path.length()
            offset = path.point(0)

            path_iter = iter(deferred_positions)
            for x in range(count):

                point = path.point(x / (endcount - 1))
                point = ((point - offset) * multiplier) + first_point

                current_position.update(_coord_to_pos(point))

                rotation = _path_angle(path, x / (endcount - 1))
                current_position['data-rotate-z'] = rotation
                yield current_position.copy()
                try:
                    position = next(path_iter)
                except StopIteration:
                    last = True
                    break
                _update_position(current_position, position)

            if last:
                break

            continue

        yield current_position.copy()
        try:
            position = next(positer)
        except StopIteration:
            break
        _update_position(current_position, position)


def update_positions(tree, positions):
    """Updates the tree with new positions"""

    for step, pos in zip(tree.findall('step'), positions):
        for key in sorted(pos):
            value = pos.get(key)
            if key.endswith("-rel"):
                abs_key = key[:key.index("-rel")]
                if value is not None:
                    els = tree.findall(".//*[@id='" + value + "']")
                    for el in els :
                        pos[abs_key] = num(el.get(abs_key)) + pos.get(abs_key)
                        step.attrib[abs_key] = str(pos.get(abs_key))
            else:
                step.attrib[key] = str(pos[key])

        if 'hovercraft-path' in step.attrib:
            del step.attrib['hovercraft-path']


def position_slides(tree):
    """Position the slides in the tree"""

    positions = gather_positions(tree)
    positions = calculate_positions(positions)
    update_positions(tree, positions)



import os
import configparser
import shutil
import glob

from lxml import etree

RESOURCE_TYPES = range(4)
CSS_RESOURCE, JS_RESOURCE, DIRECTORY_RESOURCE, OTHER_RESOURCE = RESOURCE_TYPES

JS_POSITIONS = range(2)
JS_POSITION_HEADER, JS_POSITION_BODY = JS_POSITIONS

HOVERCRAFT_DIR = os.path.split(__file__)[0]


class Resource(object):

    def __init__(self, filepath, resource_type, target=None, extra_info=None, is_in_template=False):
        self.filepath = filepath
        assert resource_type in RESOURCE_TYPES
        self.resource_type = resource_type
        if resource_type == JS_RESOURCE:
            assert extra_info in JS_POSITIONS

        self.target = target
        self.extra_info = extra_info
        self.is_in_template = is_in_template

    def final_path(self):
        if self.target is None:
            self.target = self.filepath
        if self.target.startswith('..'):
            # A path above the current path was given. Treat this as an
            # absolute path, unless it's a part of the template.
            if self.is_in_template:
                return self.target
            return os.path.abspath(self.target)

        return self.target


class Template(object):

    def __init__(self, template=None):
        self.doctype = b'<!DOCTYPE html>'
        self.resources = []

        if template is None or template in ('default', 'simple'):
            self.builtin_template = True
            if template is None:
                template = 'default'
            self.template = '/templates/%s/' % template
        else:
            self.builtin_template = False
            self.template = os.path.abspath(template)

        self._load_template_config()
        self._load_template_xsl()
        self._load_template_files()

    def add_resource(self, filepath, resource_type, target=None, extra_info=None,
                     is_in_template=False):
        self.resources.append(Resource(filepath, resource_type, target=target,
                                       extra_info=extra_info, is_in_template=is_in_template))

    def _load_template_config(self):
        if self.builtin_template:
            self.template_root = os.path.join(HOVERCRAFT_DIR, self.template.strip('/'))
        else:
            self.template_root = self.template

        if os.path.isdir(self.template_root):
            config_file = os.path.join(self.template_root, 'template.cfg')
        else:
            config_file = self.template_root
            self.template_root = os.path.split(self.template)[0]

        config = configparser.ConfigParser()
        config.read(config_file)
        self.config = config['hovercraft']

    def _load_template_files(self):

        for key, files in self.config.items():
            # CSS files:
            if key.startswith('css'):
                # This is a css_file. The media can be specified, and defaults to 'all':
                if '-' in key:
                    css, media = key.split('-')
                else:
                    media = 'all'
                for filename in files.split():
                    self.add_resource(filename, CSS_RESOURCE, extra_info=media,
                                      is_in_template=True)

            # JS files:
            elif key == 'js-header':
                for filename in files.split():
                    self.add_resource(filename, JS_RESOURCE, extra_info=JS_POSITION_HEADER,
                                      is_in_template=True)

            elif key == 'js-body':
                for filename in files.split():
                    self.add_resource(filename, JS_RESOURCE, extra_info=JS_POSITION_BODY,
                                      is_in_template=True)

            elif key == 'resource-directories':
                for filename in self.config[key].split():
                    self.add_resource(filename, DIRECTORY_RESOURCE, is_in_template=True)

            # Other files:
            elif key == 'resources':
                for filename in self.config[key].split():
                    self.add_resource(filename, OTHER_RESOURCE, is_in_template=True)

            # And finally the optional doctype:
            elif key == 'doctype':
                self.doctype = self.config['doctype'].encode()

    def _load_template_xsl(self):
        xsl_template = self.config['template']
        with open(os.path.join(self.template_root, xsl_template), 'rb') as xslfile:
            self.xsl = xslfile.read()

    def get_source_path(self, resource):
        # Non-template resource (extra css)
        if not resource.is_in_template:
            return os.path.abspath(resource.filepath)

        # In template
        if self.builtin_template:
            return os.path.join(HOVERCRAFT_DIR, self.template.strip('/'), resource.filepath)

        # External template
        return os.path.join(self.template_root, resource.filepath)

    def read_data(self, resource):
        source_path = self.get_source_path(resource)
        with open(source_path, 'rb') as infile:
            return infile.read()

    def copy_resource(self, resource, targetdir):
        """Copies a resource file and returns the source path for monitoring"""
        final_path = resource.final_path()
        if final_path[0] == '/' or (':' in final_path) or ('?' in final_path):
            # Absolute path or URI: Do nothing
            return

        source_path = self.get_source_path(resource)

        if resource.resource_type == DIRECTORY_RESOURCE:
            for file_path in glob.iglob(os.path.join(source_path, '**'), recursive=True):
                if os.path.isdir(file_path):
                    continue
                rest_target_path = file_path[len(source_path)+1:]
                target_path = os.path.join(targetdir, final_path, rest_target_path)
                # Don't yield the result, we don't monitor these.
                self._copy_file(file_path, target_path)
        else:
            target_path = os.path.join(targetdir, final_path)
            yield self._copy_file(source_path, target_path)

    def _copy_file(self, source_path, target_path):
        directory_name, filename = os.path.split(target_path)
        if not os.path.exists(directory_name):
            os.makedirs(directory_name)

        if (os.path.exists(target_path) and
            os.path.getmtime(source_path) <= os.path.getmtime(target_path)):
            # File has not changed since last copy, so skip.
            return source_path  # This file should be monitored for changes

        shutil.copy2(source_path, target_path)
        return source_path  # This file should be monitored for changes

    def copy_resources(self, targetdir):
        for resource in self.resources:
            yield from self.copy_resource(resource, targetdir)

    def xml_node(self):
        node = etree.Element('templateinfo')
        header = etree.Element('header')
        node.append(header)
        body = etree.Element('body')
        node.append(body)

        for resource in self.resources:
            if resource.resource_type == CSS_RESOURCE:
                header.append(etree.Element('css', attrib={'href': resource.final_path(),
                                                           'media': resource.extra_info}))
            elif resource.resource_type == JS_RESOURCE:
                js_element = etree.Element('js', attrib={'src': resource.final_path()})
                if resource.extra_info == JS_POSITION_BODY:
                    body.append(js_element)
                else:
                    header.append(js_element)
        return node



HTML_OUTPUTS = {

    'simple': (
        b'<!DOCTYPE html><html xmlns="http://www.w3.org/1999/xhtml"><body>'
        b'<div id="impress"><div class="step step-level-1" step="0" '
        b'data-rotate-x="0" data-rotate-y="0" data-rotate-z="0" '
        b'data-scale="1" data-x="0" data-y="0" data-z="0"><h1 '
        b'id="simple-presentation">Simple Presentation</h1><p>This '
        b'presentation has two slides, each with a '
        b'header and some text.</p></div><div class="step step-level-1" '
        b'step="1" data-rotate-x="0" data-rotate-y="0" data-rotate-z="0" '
        b'data-scale="1" data-x="1600" data-y="0" data-z="0"><h1 '
        b'id="second-slide">Second slide</h1><p>There is no positioning or '
        b'anything fancy.</p></div></div><script type="text/javascript" '
        b'src="js/impress.js"></script><script type="text/javascript" '
        b'src="js/hovercraft-minimal.js"></script></body></html>'
    ),

    'extra_css': (
        b'<!DOCTYPE html SYSTEM "about:legacy-compat"><html '
        b'xmlns="http://www.w3.org/1999/xhtml"><head><title></title><link '
        b'rel="stylesheet" href="css/style.css" media="all"></link><link '
        b'rel="stylesheet" href="css/print.css" media="print"></link><link '
        b'rel="stylesheet" href="css/impressConsole.css" '
        b'media="screen,projection"></link><link rel="stylesheet" '
        b'href="extra.css" media="all"></link><script type="text/javascript" '
        b'src="js/dummy.js"></script></head><body '
        b'class="impress-not-supported"><div id="impress"><div class="step '
        b'step-level-1" step="0" data-rotate-x="0" data-rotate-y="0" '
        b'data-rotate-z="0" data-scale="1" data-x="0" data-y="0" data-z="0">'
        b'<h1 id="simple-presentation">Simple Presentation</h1><p>This '
        b'presentation has two slides, each with a header and some text.</p>'
        b'</div><div class="step step-level-1" step="1" data-rotate-x="0" '
        b'data-rotate-y="0" data-rotate-z="0" data-scale="1" data-x="1600" '
        b'data-y="0" data-z="0"><h1 id="second-slide">Second '
        b'slide</h1><p>There is no positioning or anything '
        b'fancy.</p></div></div><div id="hovercraft-help" '
        b'class="show"><table><tr><th>Left, Down, Page Down, Space</th><td>'
        b'Next slide</td></tr><tr><th>Right, Up, Page Up</th><td>Previous '
        b'slide</td></tr><tr><th>H</th><td>Toggle this help</td>'
        b'</tr></table></div><script type="text/javascript" '
        b'src="js/impress.js"></script><script type="text/javascript" '
        b'src="js/impressConsole.js"></script><script type="text/javascript" '
        b'src="js/hovercraft.js"></script></body></html>'
    ),

    'extra_js': (
        b'<!DOCTYPE html SYSTEM "about:legacy-compat"><html '
        b'xmlns="http://www.w3.org/1999/xhtml"><head><title></title><link '
        b'rel="stylesheet" href="css/style.css" media="all"></link><link '
        b'rel="stylesheet" href="css/print.css" media="print"></link><link '
        b'rel="stylesheet" href="css/impressConsole.css" '
        b'media="screen,projection"></link><script type="text/javascript" '
        b'src="js/dummy.js"></script></head><body '
        b'class="impress-not-supported"><div id="impress"><div class="step '
        b'step-level-1" step="0" data-rotate-x="0" data-rotate-y="0" '
        b'data-rotate-z="0" data-scale="1" data-x="0" data-y="0" data-z="0">'
        b'<h1 id="simple-presentation">Simple Presentation</h1><p>This '
        b'presentation has two slides, each with a header and some text.</p>'
        b'</div><div class="step step-level-1" step="1" data-rotate-x="0" '
        b'data-rotate-y="0" data-rotate-z="0" data-scale="1" data-x="1600" '
        b'data-y="0" data-z="0"><h1 id="second-slide">Second '
        b'slide</h1><p>There is no positioning or anything '
        b'fancy.</p></div></div><div id="hovercraft-help" '
        b'class="show"><table><tr><th>Left, Down, Page Down, Space</th><td>'
        b'Next slide</td></tr><tr><th>Right, Up, Page Up</th><td>Previous '
        b'slide</td></tr><tr><th>H</th><td>Toggle this help</td>'
        b'</tr></table></div><script type="text/javascript" '
        b'src="js/impress.js"></script><script type="text/javascript" '
        b'src="js/impressConsole.js"></script><script type="text/javascript" '
        b'src="js/hovercraft.js"></script><script type="text/javascript" '
        b'src="extra.js"></script></body></html>'
    ),

    'advanced': (
        b'<!DOCTYPE html SYSTEM "about:legacy-compat"><html '
        b'xmlns="http://www.w3.org/1999/xhtml"><head><title>Presentation '
        b'title</title><link rel="stylesheet" href="css/style.css" '
        b'media="all"></link><link rel="stylesheet" href="css/print.css" '
        b'media="print"></link><link rel="stylesheet" '
        b'href="css/impressConsole.css" media="screen,projection"></link>'
        b'<link rel="stylesheet" href="extra.css" media="screen"></link>'
        b'<script type="text/javascript" src="js/dummy.js"></script></head>'
        b'<body class="impress-not-supported"><div id="impress" '
        b'data-transition-duration="2000" auto-console="True"><div '
        b'class="step step-level-1" step="0" data-x="1000" data-y="1600" '
        b'data-rotate-x="0" data-rotate-y="0" data-rotate-z="0" '
        b'data-scale="1" data-z="0"><h1 id="advanced-presentation">Advanced '
        b'Presentation</h1><p>Here we show the positioning feature, where we '
        b'can explicitly set a position\non one of the steps.</p></div><div '
        b'class="step step-level-1" step="1" id="name-this-step" '
        b'data-x="2600" data-rotate-x="0" data-rotate-y="0" data-rotate-z="0" '
        b'data-scale="1" data-y="1600" data-z="0"><h1 id="formatting">'
        b'Formatting</h1><p>Let us also try some basic formatting, like <em>'
        b'italic</em>, and <strong>bold</strong>.</p><ul><li>We can also</li>'
        b'<li>have a list</li><li>of things.</li></ul></div><div class="step '
        b'step-level-1" step="2" data-rotate-x="0" data-rotate-y="0" '
        b'data-rotate-z="0" data-scale="1" data-x="4200" data-y="1600" '
        b'data-z="0"><p>There should also be possible to have\npreformatted '
        b'text for code.</p><pre class="highlight code python"><span '
        b'class="k">def</span> <span class="nf">foo</span><span class="p">'
        b'(</span><span class="n">bar</span><span class="p">):</span>\n    '
        b'<span class="c1"># Comment</span>\n    <span class="n">a</span> '
        b'<span class="o">=</span> <span class="mi">1</span> <span class="o">'
        b'+</span> <span class="s2">"hubbub"</span>\n    <span class="k">'
        b'return</span> <span class="bp">None</span></pre></div><div '
        b'class="step step-level-1" step="3" data-rotate-x="0" '
        b'data-rotate-y="0" data-rotate-z="0" data-scale="1" data-x="5800" '
        b'data-y="1600" data-z="0"><p>An image, with attributes:</p><img '
        b'src="images/hovercraft_logo.png" width="50%" '
        b'class="imageclass"></img></div><div class="step step-level-1" '
        b'step="4" data-rotate-x="0" data-rotate-y="0" data-rotate-z="0" '
        b'data-scale="1" data-x="7400" data-y="1600" data-z="0"><h1 '
        b'id="character-sets">Character sets</h1><p>The character set is '
        b'UTF-8 as of now. Like this: &#xE5;&#xE4;&#xF6;.</p></div></div>'
        b'<div id="hovercraft-help" class="show"><table><tr><th>Left, Down, '
        b'Page Down, Space</th><td>Next slide</td></tr><tr><th>Right, Up, '
        b'Page Up</th><td>Previous slide</td></tr><tr><th>H</th><td>Toggle '
        b'this help</td></tr></table></div><script type="text/javascript" '
        b'src="js/impress.js"></script><script type="text/javascript" '
        b'src="js/impressConsole.js"></script><script type="text/javascript" '
        b'src="js/hovercraft.js"></script><script type="text/javascript" '
        b'src="extra.js"></script></body></html>'
    ),

    'default-template': (
        b'<!DOCTYPE html><html xmlns="http://www.w3.org/1999/xhtml"><head>'
        b'<title>Presentation title</title><meta charset="UTF-8"></meta><meta '
        b'name="generator" content="Hovercraft! 1.0 '
        b'http://regebro.github.com/hovercraft">'
        b'</meta><link rel="stylesheet" href="css/hovercraft.css" '
        b'media="all"></link><link rel="stylesheet" '
        b'href="css/impressConsole.css" media="all"></link>'
        b'<link rel="stylesheet" href="css/highlight.css" media="all"></link>'
        b'<link rel="stylesheet" href="extra.css" media="screen"></link>'
        b'''<script type="text/x-mathjax-config">
      MathJax.Hub.Config({
        showProcessingMessages: false,
        messageStyle: "none",
        TeX : { extensions : ['color.js'] }
      });
    </script>'''
        b'</head><body class="impress-not-supported"><div id="impress" '
        b'data-transition-duration="2000" auto-console="True"><div '
        b'class="step step-level-1" step="0" data-x="1000" data-y="1600" '
        b'data-rotate-x="0" data-rotate-y="0" data-rotate-z="0" '
        b'data-scale="1" data-z="0"><h1 id="advanced-presentation">Advanced '
        b'Presentation</h1><p>Here we show the positioning feature, where we '
        b'can explicitly set a position\non one of the steps.</p></div><div '
        b'class="step step-level-1" step="1" id="name-this-step" '
        b'data-x="2600" data-rotate-x="0" data-rotate-y="0" data-rotate-z="0" '
        b'data-scale="1" data-y="1600" data-z="0"><h1 id="formatting">'
        b'Formatting</h1><p>Let us also try some basic formatting, like '
        b'<em>italic</em>, and <strong>bold</strong>.</p><ul><li>'
        b'We can also</li><li>have a list</li><li>of things.</li></ul></div>'
        b'<div class="step step-level-1" step="2" data-rotate-x="0" '
        b'data-rotate-y="0" data-rotate-z="0" data-scale="1" data-x="4200" '
        b'data-y="1600" data-z="0"><p>There should also be possible to '
        b'have\npreformatted text for code.</p><pre class="highlight code'
        b' python"><span class="k">def</span> <span class="nf">foo</span>'
        b'<span class="p">(</span><span class="n">bar</span>'
        b'<span class="p">):</span>\n    <span class="c1">'
        b'# Comment</span>\n    <span class="n">a</span> <span class="o">='
        b'</span> <span class="mi">1</span> <span class="o">+</span> <span '
        b'class="s2">"hubbub"</span>\n    <span class="k">return</span> <span '
        b'class="bp">None</span></pre></div><div class="step step-level-1" '
        b'step="3" data-rotate-x="0" data-rotate-y="0" data-rotate-z="0" '
        b'data-scale="1" data-x="5800" data-y="1600" data-z="0"><p>An image,'
        b' with attributes:</p><img src="images/hovercraft_logo.png" '
        b'width="50%" class="imageclass"></img></div><div class="step '
        b'step-level-1" step="4" data-rotate-x="0" data-rotate-y="0" '
        b'data-rotate-z="0" data-scale="1" data-x="7400" data-y="1600" '
        b'data-z="0"><h1 id="character-sets">Character sets</h1>'
        b'<p>The character set is UTF-8 as of now. Like this: '
        b'&#xE5;&#xE4;&#xF6;.</p></div></div><div id="hovercraft-help">'
        b'<table><tr><th>Space</th><td>Forward</td></tr><tr><th>'
        b'Right, Down, Page Down</th><td>Next slide</td></tr><tr><th>'
        b'Left, Up, Page Up</th><td>Previous slide</td></tr><tr>'
        b'<th>G</th><td>Go to slide number</td></tr><tr><th>P</th>'
        b'<td>Open presenter console</td></tr><tr><th>H</th><td>Toggle '
        b'this help</td></tr></table></div><script type="text/javascript" '
        b'src="js/impress.js"></script><script type="text/javascript" '
        b'src="js/impressConsole.js"></script><script type="text/javascript" '
        b'src="js/hovercraft.js"></script><script type="text/javascript" '
        b'src="extra.js"></script></body></html>'
    ),

    'presenter-notes': (
        b'<!DOCTYPE html SYSTEM "about:legacy-compat"><html '
        b'xmlns="http://www.w3.org/1999/xhtml"><head><title>Document '
        b'title</title><link rel="stylesheet" href="css/style.css" '
        b'media="all"></link><link rel="stylesheet" href="css/print.css" '
        b'media="print"></link><link rel="stylesheet" '
        b'href="css/impressConsole.css" media="screen,projection"></link>'
        b'<script type="text/javascript" src="js/dummy.js"></script></head>'
        b'<body class="impress-not-supported"><div id="impress"><div '
        b'class="step step-level-1" step="0" data-rotate-x="0" '
        b'data-rotate-y="0" data-rotate-z="0" data-scale="1" data-x="0" '
        b'data-y="0" data-z="0"><h1 '
        b'id="hovercrafts-presenter-notes">Hovercrafts presenter notes</h1>'
        b'<p>Hovercraft! supports presenter notes. It does this by taking '
        b'anything in a\nwhat is calles a "notes-admonition" and making that '
        b'into presenter notes.</p><div class="notes"><p>Hence, this will '
        b'show up as presenter notes.\nYou have still access to a lot of '
        b'formatting, like</p><ul><li>Bullet lists</li><li>And <em>all</em> '
        b'types of <strong>inline formatting</strong></li></ul></div></div>'
        b'<div class="step step-level-1" step="1" data-rotate-x="0" '
        b'data-rotate-y="0" data-rotate-z="0" data-scale="1" data-x="1600" '
        b'data-y="0" data-z="0"><img '
        b'src="images/hovercraft_logo.png"></img><div class="notes">'
        b'<p>You don\'t have to start the text on the same line as\nthe note, '
        b'but you can.</p><p>You can also have several paragraphs. You can '
        b'not have any\nheadings of any kind though.</p><p><strong>But you '
        b'can fake them through bold-text</strong></p><p>And that\'s useful '
        b'enough for presentation notes.</p></div></div></div><div '
        b'id="hovercraft-help" class="show"><table><tr><th>Left, Down, Page '
        b'Down, Space</th><td>Next slide</td></tr><tr><th>Right, Up, Page '
        b'Up</th><td>Previous slide</td></tr><tr><th>H</th><td>Toggle this '
        b'help</td></tr></table></div><script type="text/javascript" '
        b'src="js/impress.js"></script><script type="text/javascript" '
        b'src="js/impressConsole.js"></script><script type="text/javascript" '
        b'src="js/hovercraft.js"></script></body></html>'
    ),

    'skip-presenter-notes': (
        b'<!DOCTYPE html SYSTEM "about:legacy-compat"><html '
        b'xmlns="http://www.w3.org/1999/xhtml"><head><title>Document title'
        b'</title><link rel="stylesheet" href="css/style.css" media="all">'
        b'</link><link rel="stylesheet" href="css/print.css" media="print">'
        b'</link><link rel="stylesheet" href="css/impressConsole.css" '
        b'media="screen,projection"></link><script type="text/javascript" '
        b'src="js/dummy.js"></script></head><body '
        b'class="impress-not-supported"><div id="impress"><div class="step '
        b'step-level-1" step="0" data-rotate-x="0" data-rotate-y="0" '
        b'data-rotate-z="0" data-scale="1" data-x="0" data-y="0" data-z="0">'
        b'<h1 id="hovercrafts-presenter-notes">Hovercrafts presenter notes'
        b'</h1><p>Hovercraft! supports presenter notes. It does this by '
        b'taking anything in a\nwhat is calles a "notes-admonition" and '
        b'making that into presenter notes.</p></div><div class="step '
        b'step-level-1" step="1" data-rotate-x="0" data-rotate-y="0" '
        b'data-rotate-z="0" data-scale="1" data-x="1600" data-y="0" '
        b'data-z="0"><img src="images/hovercraft_logo.png">'
        b'</img></div></div><div id="hovercraft-help" class="show"><table><tr>'
        b'<th>Left, Down, Page Down, Space</th><td>Next slide</td></tr><tr>'
        b'<th>Right, Up, Page Up</th><td>Previous slide</td></tr><tr><th>H'
        b'</th><td>Toggle this help</td></tr></table></div><script '
        b'type="text/javascript" src="js/impress.js"></script><script '
        b'type="text/javascript" src="js/impressConsole.js"></script><script '
        b'type="text/javascript" src="js/hovercraft.js"></script></body>'
        b'</html>'
    ),

    'table': (
        b'<!DOCTYPE html><html xmlns="http://www.w3.org/1999/xhtml">'
        b'<body><div id="impress"><div class="step step-level-1" step="0" '
        b'data-rotate-x="0" data-rotate-y="0" data-rotate-z="0" '
        b'data-scale="1" data-x="0" data-y="0" data-z="0"><table '
        b'cellpadding="0" cellspacing="0" class="my-table-class">Truth table '
        b'for "not"<thead><tr><th><p>Name</p></th><th><p>Money Owed</p></th>'
        b'</tr></thead><tbody><tr><td><p>Adam Alpha</p></td><td><p>100</p>'
        b'</td></tr></tbody></table><table cellpadding="0" cellspacing="0" '
        b'id="my-table"><thead><tr><th><p>Number</p></th><th><p>Two</p></th>'
        b'</tr></thead><tbody><tr><td><p>Adam Alpha</p></td><td><p>100</p>'
        b'</td></tr></tbody></table></div></div><script '
        b'type="text/javascript" src="js/impress.js"></script><script '
        b'type="text/javascript" src="js/hovercraft-minimal.js"></script>'
        b'</body></html>'
    ),

    'slide_with_class': (
        b'<!DOCTYPE html><html xmlns="http://www.w3.org/1999/xhtml"><body>'
        b'<div id="impress"><div class="step step-level-1 something-else" '
        b'step="0" data-rotate-x="0" data-rotate-y="0" data-rotate-z="0" '
        b'data-scale="1" data-x="0" data-y="0" data-z="0"><p>This is some '
        b'text</p></div></div><script type="text/javascript" '
        b'src="js/impress.js"></script><script type="text/javascript" '
        b'src="js/hovercraft-minimal.js"></script></body></html>'
    ),

    'container_directive': (
        b'<!DOCTYPE html><html xmlns="http://www.w3.org/1999/xhtml"><body>'
        b'<div id="impress"><div class="step step-level-1" step="0" '
        b'data-rotate-x="0" data-rotate-y="0" data-rotate-z="0" '
        b'data-scale="1" data-x="0" data-y="0" data-z="0"><div '
        b'class="my-class"><p>This is some text in the container</p></div>'
        b'<div id="my-thing"><p>This should have an id</p></div></div></div>'
        b'<script type="text/javascript" src="js/impress.js"></script><script '
        b'type="text/javascript" src="js/hovercraft-minimal.js"></script>'
        b'</body></html>'
    ),

    'class_directive': (
        b'<!DOCTYPE html><html xmlns="http://www.w3.org/1999/xhtml"><body>'
        b'<div id="impress"><div class="step step-level-1" step="0" '
        b'data-rotate-x="0" data-rotate-y="0" data-rotate-z="0" '
        b'data-scale="1" data-x="0" data-y="0" data-z="0"><p class="my-class">'
        b'This is some text in the class</p></div></div><script '
        b'type="text/javascript" src="js/impress.js"></script><script '
        b'type="text/javascript" src="js/hovercraft-minimal.js"></script>'
        b'</body></html>'
    ),

    'comment': (
        b'<!DOCTYPE html><html xmlns="http://www.w3.org/1999/xhtml"><body>'
        b'<div id="impress"><div class="step step-level-1" step="0" '
        b'data-rotate-x="0" data-rotate-y="0" data-rotate-z="0" '
        b'data-scale="1" data-x="0" data-y="0" data-z="0"><p>This text '
        b'should appear.</p></div></div><script type="text/javascript" '
        b'src="js/impress.js"></script><script type="text/javascript" '
        b'src="js/hovercraft-minimal.js"></script></body></html>'
    )
}



from docutils import nodes
from docutils.core import publish_string
from docutils.readers.standalone import Reader
from docutils.transforms.misc import Transitions
from docutils.writers.docutils_xml import Writer
from lxml import etree


class HovercraftTransitions(Transitions):

    @property
    def _doc_transitions(self):
        if not hasattr(self.document, '_transitions'):
            self.document._transitions = []
        return self.document._transitions

    def visit_transition(self, node):
        # First add the level of the transition as a class
        char = node.rawsource[0]
        if char not in self._doc_transitions:
            self._doc_transitions.append(char)
        level = self._doc_transitions.index(char) + 1
        node.attributes['level'] = str(level)

        index = node.parent.index(node)
        error = None
        # Here the default transformer complained if you had a transition as
        # the first thing in a section. I don't see why.
        if isinstance(node.parent[index - 1], nodes.transition):
            error = self.document.reporter.error(
                'At least one body element must separate transitions; '
                'adjacent transitions are not allowed.',
                source=node.source, line=node.line)
        if error:
            # Insert before node and update index.
            node.parent.insert(index, error)
            index += 1
        assert index < len(node.parent)
        if index != len(node.parent) - 1:
            # No need to move the node.
            return
        # Node behind which the transition is to be moved.
        sibling = node
        # While sibling is the last node of its parent.
        while index == len(sibling.parent) - 1:
            sibling = sibling.parent
            # If sibling is the whole document (i.e. it has no parent).
            if sibling.parent is None:
                # Transition at the end of document.  Do not move the
                # transition up, and place an error behind.
                error = self.document.reporter.error(
                    'Document may not end with a transition.',
                    line=node.line)
                node.parent.insert(node.parent.index(node) + 1, error)
                return
            index = sibling.parent.index(sibling)
        # Remove the original transition node.
        node.parent.remove(node)
        # Insert the transition after the sibling.
        sibling.parent.insert(index + 1, node)


class HovercraftReader(Reader):

    def get_transforms(self):
        transforms = Reader().get_transforms()
        transforms.remove(Transitions)
        transforms.append(HovercraftTransitions)
        return transforms


def rst2xml(rststring, source_path=None):
    reader = HovercraftReader()
    writer = Writer()
    result =  publish_string(rststring,
                          source_path=source_path,
                          reader=reader,
                          writer=writer,
                          settings_overrides={'syntax_highlight': 'short'})
    dependencies = reader.settings.record_dependencies.list
    return result, dependencies


def copy_node(node):
    """Makes a copy of a node with the same attributes and text, but no children."""
    element = node.makeelement(node.tag)
    element.text = node.text
    element.tail = node.tail
    for key, value in node.items():
        element.set(key, value)

    return element


class SlideMaker(object):
    """A docutils XML walker that will organize the XML into slides"""

    def __init__(self, intree, skip_notes=False):
        self.intree = intree
        self.result = None
        self.curnode = None
        self.steps = 0
        self.skip_notes = skip_notes
        self.skip_nodes = ('docinfo', 'field_list', 'field', 'field_body',)
        self.need_mathjax = False

    def _newstep(self, level):
        step = etree.Element('step', attrib={
            'step': str(self.steps),
            'class': 'step step-level-%s' % level,
        })
        self.steps += 1
        return step

    def walk(self):
        walken = etree.iterwalk(self.intree, events=('start', 'end'))
        for event, node in walken:
            if node.tag in self.skip_nodes:
                continue
            method = getattr(self, '%s_%s' % (event, node.tag), None)
            if method is None:
                if event == 'start':
                    self.default_start(node)
                else:
                    if event == 'end':
                        self.default_end(node)
            else:
                method(node)

        return self.result

    def default_start(self, node):
        new = copy_node(node)
        self.curnode.append(new)
        self.curnode = new

    def default_end(self, node):
        if self.curnode.tag != 'step':
            self.curnode = self.curnode.getparent()

    def start_document(self, node):
        self.curnode = self.result = copy_node(node)

    def start_section(self, node):
        # If there has been no transition by now, start a new step:
        if self.steps == 0:
            step = self._newstep(1)
            self.result.append(step)
            self.curnode = step

        # Then carry on as normal
        self.default_start(node)

    def start_transition(self, node):
        level = int(node.attrib['level'])
        step = self._newstep(level)
        # Walk back up to a transition of the same level
        curnode = self.curnode
        def get_level(clss):
            for cls in clss.split():
                # Skip unrelated classes, such as those added by the user.
                if cls.startswith('step-level-'):
                    return int(cls.split('-')[-1])
        while (curnode.tag != 'step' or
               get_level(curnode.attrib['class']) >= level):
            parent = curnode.getparent()
            if parent is None:
                # Top of tree
                break
            curnode = parent

        # This is the transition above the new transition.
        # Add the new transition to the end.
        curnode.append(step)
        self.curnode = step

    def end_transition(self, node):
        self.default_end(node)

    def start_field_name(self, node):
        # Fields are made into attribute, nothing to do here:.
        pass

    def end_field_name(self, node):
        # Fields are made into attributes
        pass

    def start_paragraph(self, node):
        # Fields are made into attributes.
        parent = node.getparent()
        if parent.tag == 'field_body':
            fieldname = parent.getprevious().text
            current = self.curnode.get(fieldname)
            if current:
                value = current + ' ' + node.text
            else:
                value = node.text
            self.curnode.set(fieldname, value)
        else:
            self.default_start(node)

    def end_paragraph(self, node):
        parent = node.getparent()
        if parent.tag != 'field_body':
            self.default_end(node)

    def start_note(self, node):
        if not self.skip_notes:
            return self.default_start(node)
        # Skip this node completely, including children:
        while len(node) > 0:
            del node[0]

    def start_math_block(self, node):
        self.need_mathjax = True
        self.default_start(node)

    def start_math(self, node):
        self.need_mathjax = True
        self.default_start(node)


import unittest
from pkg_resources import resource_string
from lxml import etree

from hovercraft.parse import SlideMaker, rst2xml


def make_tree(file_name):
    """Loads reStructuredText, outputs an lxml tree"""
    rst = resource_string(__name__, file_name)
    xml, dependencies = rst2xml(rst)
    return etree.fromstring(xml)


class SlideMakerTests(unittest.TestCase):
    """Test the conversion of docutils XML into XML suitable to give to the templates"""

    def test_simple(self):
        tree = SlideMaker(make_tree('test_data/simple.rst')).walk()
        self.assertEqual(etree.tostring(tree), (
            b'<document source="&lt;string&gt;"><step class="step step-level-1" step="0">'
            b'<section ids="simple-presentation" names="simple\\ presentation">'
            b'<title>Simple Presentation</title><paragraph>This presentation '
            b'has two slides, each with a header and some text.</paragraph>'
            b'</section></step><step class="step step-level-1" step="1"><section '
            b'ids="second-slide" names="second\\ slide"><title>Second slide</title><paragraph>'
            b'There is no positioning or anything fancy.</paragraph>'
            b'</section></step></document>'))

    def test_advanced(self):
        tree = SlideMaker(make_tree('test_data/advanced.rst')).walk()
        xml = etree.tostring(tree)
        target = (
            b'<document source="&lt;string&gt;" title="Presentation title" '
            b'data-transition-duration="2000" auto-console="True" '
            b'css-screen="extra.css" js-body="extra.js"><paragraph>This is an advanced '
            b'presentation. It doesn\'t have a section in the first\nstep, '
            b'meaning the first step will not be a step at all, but a sort of\n'
            b'introductory comment about the presentation, that will not show up '
            b'in the\npresentation at all.</paragraph><paragraph>It also sets a '
            b'title and a transition-duration.</paragraph><step class="step '
            b'step-level-1" step="0" data-x="1000" data-y="1600"><section '
            b'ids="advanced-presentation" names="advanced\\ presentation">'
            b'<title>Advanced Presentation</title><paragraph>Here we '
            b'show the positioning feature, where we can explicitly set a '
            b'position\non one of the steps.</paragraph></section></step><step '
            b'class="step step-level-1" step="1" id="name-this-step" '
            b'data-x="r1600"><section ids="formatting" names="formatting"><title>'
            b'Formatting</title><paragraph>Let us also try some basic '
            b'formatting, like <emphasis>italic</emphasis>, and <strong>bold'
            b'</strong>.</paragraph><bullet_list bullet="*"><list_item>'
            b'<paragraph>We can also</paragraph></list_item><list_item>'
            b'<paragraph>have a list</paragraph></list_item><list_item>'
            b'<paragraph>of things.</paragraph></list_item></bullet_list>'
            b'</section></step><step class="step step-level-1" step="2">'
            b'<paragraph>There should also be possible to have\n'
            b'preformatted text for code.</paragraph><literal_block '
            b'classes="code python" xml:space="preserve"><inline classes="k">'
            b'def</inline> <inline classes="nf">foo</inline><inline '
            b'classes="p">(</inline><inline classes="n">bar</inline><inline '
            b'classes="p">):</inline>\n    <inline classes="c1"># Comment'
            b'</inline>\n    <inline classes="n">a</inline> <inline '
            b'classes="o">=</inline> <inline classes="mi">1</inline> <inline '
            b'classes="o">+</inline> <inline classes="s2">"hubbub"</inline>'
            b'\n    <inline classes="k">return</inline> <inline classes="bp">'
            b'None</inline></literal_block></step><step class="step step-level-1"'
            b' step="3"><paragraph>An image, with attributes:</paragraph><image '
            b'classes="imageclass" uri="images/hovercraft_logo.png" width="50%"/>'
            b'</step><step class="step step-level-1" step="4"><section ids="character-sets" '
            b'names="character\\ sets"><title>Character sets</title><paragraph>'
            b'The character set is UTF-8 as of now. Like this: '
            b'&#229;&#228;&#246;.</paragraph></section></step></document>')
        self.assertEqual(xml, target)

    def test_presenter_notes(self):
        tree = SlideMaker(make_tree('test_data/presenter-notes.rst')).walk()
        target = (
            b'<document ids="document-title" names="document\\ title" '
            b'source="&lt;string&gt;" title="Document title"><title>Document '
            b'title</title><step '
            b'class="step step-level-1" step="0"><section ids="hovercrafts-presenter-notes" '
            b'names="hovercrafts\\ presenter\\ notes"><title>Hovercrafts presenter '
            b'notes</title><paragraph>Hovercraft! supports presenter notes. It does '
            b'this by taking anything in a\nwhat is calles a "notes-admonition" and '
            b'making that into presenter notes.</paragraph><note><paragraph>Hence, '
            b'this will show up as presenter notes.\nYou have still access to a lot '
            b'of formatting, like</paragraph><bullet_list bullet="*"><list_item>'
            b'<paragraph>Bullet lists</paragraph></list_item><list_item><paragraph>'
            b'And <emphasis>all</emphasis> types of <strong>inline formatting'
            b'</strong></paragraph></list_item></bullet_list></note></section>'
            b'</step><step class="step step-level-1" step="1"><image '
            b'uri="images/hovercraft_logo.png"/><note><paragraph>You '
            b'don\'t have to start the text on the same line as\nthe note, but '
            b'you can.</paragraph><paragraph>You can also have several paragraphs.'
            b' You can not have any\nheadings of any kind '
            b'though.</paragraph><paragraph><strong>But you can fake them through '
            b'bold-text</strong></paragraph><paragraph>And that\'s useful enough '
            b'for presentation notes.</paragraph></note></step></document>')
        self.assertEqual(etree.tostring(tree), target)

    def test_transition_levels(self):
        # Make the XML
        xml, deps = rst2xml(
            b'Intro\n\n====\n\nLevel 1\n\n====\n\nLevel 1\n\n----\n\nLevel 2\n\n'
            b'....\n\nLevel 3\n\n----\n\nLevel 2\n\n....\n\nLevel 3\n\n'
            b'====\n\nLevel 1')

        target_start = (
            b'<?xml version="1.0" encoding="utf-8"?>\n<!DOCTYPE document PUBLIC "+'
            b'//IDN docutils.sourceforge.net//DTD Docutils Generic//EN//XML" '
            b'"http://docutils.sourceforge.net/docs/ref/docutils.dtd">\n')
        target_end = (
            b'<document source="&lt;string&gt;"><paragraph>Intro</paragraph>'
            b'<transition level="1"></transition><paragraph>Level 1</paragraph>'
            b'<transition level="1"></transition><paragraph>Level 1</paragraph>'
            b'<transition level="2"></transition><paragraph>Level 2</paragraph>'
            b'<transition level="3"></transition><paragraph>Level 3</paragraph>'
            b'<transition level="2"></transition><paragraph>Level 2</paragraph>'
            b'<transition level="3"></transition><paragraph>Level 3</paragraph>'
            b'<transition level="1"></transition><paragraph>Level 1</paragraph>'
            b'</document>')
        self.assertTrue(xml.startswith(target_start))
        self.assertTrue(xml.endswith(target_end))

        # Make the slides:
        tree = SlideMaker(etree.fromstring(xml)).walk()

        target = (
            b'<document source="&lt;string&gt;"><paragraph>Intro</paragraph>'
            b'<step class="step step-level-1" step="0"><paragraph>Level 1</paragraph></step>'
            b'<step class="step step-level-1" step="1"><paragraph>Level 1</paragraph>'
            b'<step class="step step-level-2" step="2"><paragraph>Level 2</paragraph>'
            b'<step class="step step-level-3" step="3"><paragraph>Level 3</paragraph></step>'
            b'</step><step class="step step-level-2" step="4"><paragraph>Level 2</paragraph>'
            b'<step class="step step-level-3" step="5"><paragraph>Level 3</paragraph></step>'
            b'</step></step><step class="step step-level-1" step="6"><paragraph>Level 1'
            b'</paragraph></step></document>'
        )
        self.assertEqual(etree.tostring(tree), target)

if __name__ == '__main__':
    unittest.main()



import os
import unittest
from lxml import etree

from hovercraft.template import (Template, CSS_RESOURCE, JS_RESOURCE,
                                 JS_POSITION_BODY, JS_POSITION_HEADER)

TEST_DATA = os.path.join(os.path.split(__file__)[0], 'test_data')


class TemplateInfoTests(unittest.TestCase):
    """Tests that template information is correctly parsed"""

    def test_template_paths(self):
        # You can specify a folder or a cfg file and that's the same thing.
        template_info1 = Template(os.path.join(TEST_DATA, 'minimal'))
        template_info2 = Template(os.path.join(TEST_DATA, 'minimal', 'template.cfg'))
        self.assertEqual(etree.tostring(template_info1.xml_node()),
                         etree.tostring(template_info2.xml_node()))

    def test_template_minimal(self):
        template_info = Template(os.path.join(TEST_DATA, 'minimal'))
        with open(os.path.join(TEST_DATA, 'minimal', 'template.xsl'), 'rb') as xslfile:
            xsl = xslfile.read()
        self.assertEqual(template_info.xsl, xsl)
        template_files = [each.filepath for each in template_info.resources]
        self.assertIn('js/impress.js', template_files)
        self.assertIn('js/hovercraft-minimal.js', template_files)
        css_files = list(each.filepath for each in template_info.resources if
                         each.resource_type == CSS_RESOURCE)
        self.assertEqual(len(css_files), 0)
        self.assertEqual(template_info.doctype, b'<!DOCTYPE html>')

    def test_template_maximal(self):
        template_info = Template(os.path.join(TEST_DATA, 'maximal'))
        with open(os.path.join(TEST_DATA, 'maximal', 'template.xsl'), 'rb') as xslfile:
            xsl = xslfile.read()
        self.assertEqual(template_info.xsl, xsl)

        template_files = [each.filepath for each in template_info.resources]
        self.assertIn('images/hovercraft_logo.png', template_files)
        self.assertIn('js/impress.js', template_files)
        self.assertIn('js/impressConsole.js', template_files)
        self.assertIn('js/hovercraft.js', template_files)

        js_bodies = [each.filepath for each in template_info.resources if
                     each.resource_type == JS_RESOURCE and
                     each.extra_info == JS_POSITION_BODY]
        self.assertIn('js/impress.js', js_bodies)
        self.assertIn('js/impressConsole.js', js_bodies)
        self.assertIn('js/hovercraft.js', js_bodies)

        js_headers = [each.filepath for each in template_info.resources if
                      each.resource_type == JS_RESOURCE and
                      each.extra_info == JS_POSITION_HEADER]
        self.assertIn('js/dummy.js', js_headers)

        self.assertEqual(template_info.resources[0].filepath, 'css/style.css')
        self.assertEqual(template_info.resources[0].extra_info, 'all')
        self.assertEqual(template_info.resources[1].filepath, 'css/print.css')
        self.assertEqual(template_info.resources[1].extra_info, 'print')
        self.assertEqual(template_info.resources[2].filepath, 'css/impressConsole.css')
        self.assertEqual(template_info.resources[2].extra_info, 'screen,projection')

        self.assertEqual(template_info.doctype, b'<!DOCTYPE html SYSTEM "about:legacy-compat">')


class TemplateInfoNodeTests(unittest.TestCase):
    """Tests that template information is correctly made into an xml nodes"""

    def test_minimal_template(self):
        template_info = Template(os.path.join(TEST_DATA, 'minimal'))
        node = template_info.xml_node()

        self.assertEqual(etree.tostring(node), (
            b'<templateinfo><header/><body>'
            b'<js src="js/impress.js"/><js src="js/hovercraft-minimal.js"/>'
            b'</body></templateinfo>'))

    def test_maximal_template(self):
        template_info = Template(os.path.join(TEST_DATA, 'maximal'))
        node = template_info.xml_node()

        self.assertEqual(etree.tostring(node), (
            b'<templateinfo><header>'
            b'<css href="css/style.css" media="all"/>'
            b'<css href="css/print.css" media="print"/>'
            b'<css href="css/impressConsole.css" media="screen,projection"/>'
            b'<js src="js/dummy.js"/></header>'
            b'<body><js src="js/impress.js"/><js src="js/impressConsole.js"/>'
            b'<js src="js/hovercraft.js"/>'
            b'</body></templateinfo>'))


if __name__ == '__main__':
    unittest.main()



import argparse
import gettext
import os
import sys
import threading
import time
import pkg_resources
from collections import defaultdict
from http.server import HTTPServer, SimpleHTTPRequestHandler
from tempfile import TemporaryDirectory
from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler

from .generate import generate

__version__ = pkg_resources.require("hovercraft")[0].version


class HovercraftEventHandler(FileSystemEventHandler):
    def __init__(self, filelist):
        self.filelist = filelist
        self.quit = False
        super().__init__()

    def on_modified(self, event):
        self._update(event.src_path)

    def on_created(self, event):
        self._update(event.src_path)

    def on_moved(self, event):
        self._update(event.dest_path)

    def _update(self, src_path):
        if self.quit:
            return
        if src_path in self.filelist:
            print("File %s modified, update presentation" % src_path)
            self.quit = True


def generate_and_observe(args, event):
    while event.isSet():
        # Generate the presentation
        monitor_list = generate(args)
        print("Presentation generated.")

        # Make a list of involved directories
        directories = defaultdict(list)
        for file in monitor_list:
            directory, filename = os.path.split(file)
            directories[directory].append(filename)

        observer = Observer()
        handler = HovercraftEventHandler(monitor_list)
        for directory, files in directories.items():
            observer.schedule(handler, directory, recursive=False)

        observer.start()
        while event.wait(1):
            time.sleep(0.05)
            if handler.quit:
                break

        observer.stop()
        observer.join()


def main(args=None):
    parser = create_arg_parser()
    args = parser.parse_args(args=args)
    serve_presentation(args)


def create_arg_parser():
    # That the argparse default strings are lowercase is ugly.

    def my_gettext(s):
        return s.capitalize()

    gettext.gettext = my_gettext

    parser = argparse.ArgumentParser(
        description='Create impress.js presentations with reStructuredText',
        add_help=False)
    parser.add_argument(
        'presentation',
        metavar='<presentation>',
        help='The path to the reStructuredText presentation file.')
    parser.add_argument(
        'targetdir',
        metavar='<targetdir>',
        nargs='?',
        help=('The directory where the presentation is saved. Will be created '
              'if it does not exist. If you do not specify a targetdir '
              'Hovercraft! will instead start a webserver and serve the '
              'presentation from that server.'))
    parser.add_argument(
        '-h', '--help',
        action='help',
        help='Show this help.')
    parser.add_argument(
        '-t',
        '--template',
        help=('Specify a template. Must be a .cfg file, or a directory with a '
              'template.cfg file. If not given it will use a default template.'))
    parser.add_argument(
        '-c',
        '--css',
        help=('An additional css file for the presentation to use. '
              'See also the ``:css:`` settings of the presentation.'))
    parser.add_argument(
        '-j',
        '--js',
        help=('An additional javascript file for the presentation to use. Added as a js-body script.'
              'See also the ``:js-body:`` settings of the presentation.'))
    parser.add_argument(
        '-a',
        '--auto-console',
        action='store_true',
        help=('Open the presenter console automatically. This is useful when '
              'you are rehearsing and making sure the presenter notes are '
              'correct. You can also set this by having ``:auto-console: '
              'true`` first in the presentation.'))
    parser.add_argument(
        '-s',
        '--skip-help',
        action='store_true',
        help=('Do not show the initial help popup.'))
    parser.add_argument(
        '-n',
        '--skip-notes',
        action='store_true',
        help=('Do not include presenter notes in the output.'))
    parser.add_argument(
        '-p',
        '--port',
        default='0.0.0.0:8000',
        help=('The address and port that the server uses. '
              'Ex 8080 or 127.0.0.1:9000. Defaults to 0.0.0.0:8000.'))
    parser.add_argument(
        '--mathjax',
        default=os.environ.get('HOVERCRAFT_MATHJAX', 'https://cdnjs.cloudflare.com/ajax/libs/mathjax/2.7.1/MathJax.js?config=TeX-MML-AM_CHTML'),
        help=('The URL to the mathjax library.'
              ' (It will only be used if you have rST ``math::`` in your document)'))
    parser.add_argument(
        '-N',
        '--slide-numbers',
        action='store_true',
        help=('Show slide numbers during the presentation.'))
    parser.add_argument(
        '-v',
        '--version',
        action='version',
        #help=('Display version and exit.'),
        version="Hovercraft! %s" % __version__
    )

    return parser


def serve_presentation(args):

    # XXX Bit of a hack, clean this up, I check for this twice, also in the template.
    if args.template and args.template not in ('simple', 'default'):
        args.template = os.path.abspath(args.template)

    if args.targetdir:
        # Generate the presentation
        generate(args)
    else:
        # Server mode. Start a server that serves a temporary directory.

        with TemporaryDirectory() as targetdir:
            args.targetdir = targetdir
            args.presentation = os.path.abspath(args.presentation)

            # Set up watchdog to regenerate presentation if saved.
            event = threading.Event()
            event.set()
            thread = threading.Thread(target=generate_and_observe, args=(args, event))
            try:
                # Serve presentation
                if ':' in args.port:
                    bind, port = args.port.split(':')
                else:
                    bind, port = '0.0.0.0', args.port
                port = int(port)

                # First create the server. This checks that we can connect to
                # the port we want to.
                os.chdir(targetdir)
                server = HTTPServer((bind, port), SimpleHTTPRequestHandler)
                print("Serving HTTP on", bind, "port", port, "...")

                try:
                    # Now generate the presentation
                    thread.start()

                    try:
                        # All is good, start the server
                        server.serve_forever()
                    except KeyboardInterrupt:
                        print("\nKeyboard interrupt received, exiting.")
                    finally:
                        # Server exited
                        server.server_close()

                finally:
                    # Stop the generation thread
                    event.clear()
                    # Wait for it to end
                    thread.join()

            except PermissionError:
                print("Can't bind to port %s:%s: No permission" % (bind, port))
            except OSError as e:
                if e.errno == 98:
                    print("Can't bind to port %s:%s: port already in use" % (bind, port))
                else:
                    raise



from io import open
from setuptools import setup, find_packages
from setuptools.command.install import install
import sys

version = '2.6'

with open('README.rst', 'rt', encoding='utf8') as readme:
    description = readme.read()

with open('CHANGES.txt', 'rt', encoding='utf8') as changes:
    history = changes.read()

class CustomInstall(install):
    def initialize_options(self):
        if sys.version < '3':
            print("Hovercraft requires Python 3.5 or higher.")
            sys.exit(1)

        return install.initialize_options(self)

setup(name='hovercraft',
      version=version,
      description="Makes impress.js presentations from reStructuredText",
      long_description=description + '\n' + history,
      # Get strings from http://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=['Development Status :: 5 - Production/Stable',
                   'Topic :: Multimedia :: Graphics :: Presentation',
                   'Topic :: Text Processing',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python :: 3',
                   'Programming Language :: Python :: 3 :: Only',
                   'Programming Language :: Python :: 3.5',
                   'Programming Language :: Python :: 3.6',
                   'License :: OSI Approved :: MIT License',
                   ],
      keywords='presentations restructuredtext',
      author='Lennart Regebro',
      author_email='regebro@gmail.com',
      url='https://github.com/regebro/hovercraft',
      license='MIT',
      packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
      include_package_data=True,
      zip_safe=False,
      install_requires=[
          'setuptools',
          'docutils >= 0.9',
          'lxml>=3.1.0',
          'svg.path',
          'pygments',
          'watchdog',
      ],
      tests_require=[
          'manuel',
      ],
      test_suite='tests',
      entry_points={
               'console_scripts': [
                   'hovercraft = hovercraft:main',
               ],
      },
      cmdclass={'install': CustomInstall}
)



import manuel.doctest
import manuel.codeblock
import manuel.testing
import unittest


def additional_tests():
    m = manuel.doctest.Manuel()
    m += manuel.codeblock.Manuel()
    return manuel.testing.TestSuite(m, r'../docs/examples/tutorial.rst')

if __name__ == '__main__':
    unittest.TextTestRunner().run(additional_tests())



import os
import unittest
from pkg_resources import resource_string
from lxml import etree

from hovercraft.parse import rst2xml, SlideMaker
from hovercraft.position import gather_positions, calculate_positions, position_slides

TEST_DATA = os.path.join(os.path.split(__file__)[0], 'test_data')


def make_tree(file_name):
    """Loads reStructuredText, outputs an lxml tree"""
    rst = resource_string(__name__, os.path.join('test_data', file_name))
    xml, deps = rst2xml(rst)
    return SlideMaker(etree.fromstring(xml)).walk()


class GatherTests(unittest.TestCase):
    """Tests that position information is correctly parsed"""

    def test_gathering(self):
        tree = make_tree('positioning.rst')

        positions = list(gather_positions(tree))

        self.assertEqual(positions, [
            {'data-x': 'r0', 'data-y': 'r0', 'data-z': 'r0',
             'data-rotate-x': 'r0', 'data-rotate-y': 'r0',
             'data-rotate-z': 'r0', 'data-scale': '1', 'is_path': False},
            {'data-x': 'r1600', 'data-y': 'r0', 'data-z': 'r0',
             'data-rotate-x': 'r0', 'data-rotate-y': 'r0',
             'data-rotate-z': 'r0', 'data-scale': 'r0', 'is_path': False},
            {'data-x': 'r1600', 'data-y': 'r0', 'data-z': 'r0',
             'data-rotate-x': 'r0', 'data-rotate-y': 'r0',
             'data-rotate-z': 'r0', 'data-scale': 'r0', 'is_path': True,
             'path': 'm 100 100 l 200 0 l 0 200'},
            {'data-x': 'r1600', 'data-y': 'r0', 'data-z': 'r0',
             'data-rotate-x': 'r0', 'data-rotate-y': 'r0',
             'data-rotate-z': 'r0', 'data-scale': 'r0', 'is_path': True},
            {'data-x': 'r1600', 'data-y': 'r0', 'data-z': 'r0',
             'data-rotate-x': 'r0', 'data-rotate-y': 'r0',
             'data-rotate-z': 'r0', 'data-scale': 'r0', 'is_path': True},
            {'data-x': '0', 'data-y': '0', 'data-z': 'r0',
             'data-rotate-x': 'r0', 'data-rotate-y': 'r0',
             'data-rotate-z': 'r0', 'data-scale': 'r0', 'is_path': False},
            {'data-x': 'r0', 'data-y': 'r0', 'data-z': 'r0',
             'data-rotate-x': 'r0', 'data-rotate-y': 'r0',
             'data-rotate-z': '90', 'data-scale': 'r0', 'is_path': False},
            {'data-x': 'r0', 'data-y': 'r0', 'data-z': 'r0',
             'data-rotate-x': 'r0', 'data-rotate-y': 'r0',
             'data-rotate-z': 'r0', 'data-scale': 'r0', 'is_path': False},
            {'data-x': 'r0', 'data-y': 'r0', 'data-z': 'r0',
             'data-rotate-x': 'r0', 'data-rotate-y': 'r0',
             'data-rotate-z': 'r0', 'data-scale': 'r0', 'is_path': True,
             'path': 'm 100 100 l 200 0 l 0 200'},
            {'data-x': 'r0', 'data-y': 'r0', 'data-z': 'r0',
             'data-rotate-x': 'r0', 'data-rotate-y': 'r0',
             'data-rotate-z': 'r0', 'data-scale': 'r0', 'is_path': True},
            {'data-x': 'r0', 'data-y': 'r0', 'data-z': '1000',
             'data-rotate-x': '180', 'data-rotate-y': 'r0',
             'data-rotate-z': 'r0', 'data-scale': 'r0', 'is_path': True},
            {'data-x': '3000', 'data-y': '1000', 'data-z': 'r0',
             'data-rotate-x': 'r0', 'data-rotate-y': 'r0',
             'data-rotate-z': 'r0', 'data-scale': 'r0', 'is_path': False},
             {'data-x': 'firstID+1000', 'data-y': 'firstID-500', 'data-z': 'r0',
             'data-rotate-x': 'r0', 'data-rotate-y': 'r0',
             'data-rotate-z': 'r0', 'data-scale': 'r0', 'is_path': False},
             {'data-x': 'secondID+800', 'data-y': '200', 'data-z': 'r0',
             'data-rotate-x': 'r0', 'data-rotate-y': 'r0',
             'data-rotate-z': 'r0', 'data-scale': 'r0', 'is_path': False}
        ])


class CalculateTests(unittest.TestCase):
    """Tests that positions are correctly calculated"""

    def test_square(self):
        # Slides, positioned in a square
        positions = [
            {'data-x': '0', 'data-y': '0'},
            {'data-x': 'r1200', 'data-y': '0'},
            {'data-x': 'r1200', 'data-y': '0'},
            {'data-x': 'r1200', 'data-y': '0'},
            {'data-x': 'r0', 'data-y': 'r-1000'},
            {'data-x': 'r0', 'data-y': 'r-1000'},
            {'data-x': 'r0', 'data-y': 'r-1000'},
            {'data-x': 'r-1200', 'data-y': 'r0'},
            {'data-x': 'r-1200', 'data-y': 'r0'},
            {'data-x': 'r-1200', 'data-y': 'r0'},
            {'data-x': 'r0', 'data-y': 'r1000'},
            {'data-x': 'r0', 'data-y': 'r1000'},
        ]

        positions = list(calculate_positions(positions))

        self.assertEqual(positions, [
            {'data-x': 0, 'data-y': 0, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 1200, 'data-y': 0, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 2400, 'data-y': 0, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 3600, 'data-y': 0, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 3600, 'data-y': -1000, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 3600, 'data-y': -2000, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 3600, 'data-y': -3000, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 2400, 'data-y': -3000, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 1200, 'data-y': -3000, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 0, 'data-y': -3000, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 0, 'data-y': -2000, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 0, 'data-y': -1000, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
        ])

    def test_relative_positioning(self):
        # Relative positioning is probably the most useful positioning.
        # It allows you to insert or remove a slide, and everything adjusts.
        positions = [
            # The first two slides are just default positons
            {'data-x': 'r0', 'data-y': 'r0'},
            {'data-x': 'r1600', 'data-y': 'r0'},
            # Then suddenly we move vertically!
            {'data-x': 'r0', 'data-y': 'r1000'},
            # Continue the same way one slide.
            {'data-x': 'r0', 'data-y': 'r1000'},
            # Stand still
            {'data-x': 'r0', 'data-y': 'r0'},
            # Stand still again!
            {'data-x': 'r0', 'data-y': 'r0'},
            # Move a little bit
            {'data-x': 'r-40', 'data-y': 'r-200'},
            # Go back to normal movement to the right
            {'data-x': 'r1600', 'data-y': 'r0'},
            {'data-x': 'r1600', 'data-y': 'r0'},
            {'data-x': 'r1600', 'data-y': 'r0'},
            # Absolute movement back to start!
            {'data-x': '0', 'data-y': '0'},
            # Absolute movement to a center for end (with zoomout for example)
            {'data-x': '3000', 'data-y': '1000'},
        ]

        positions = list(calculate_positions(positions))

        self.assertEqual(positions, [
            {'data-x': 0, 'data-y': 0, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 1600, 'data-y': 0, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 1600, 'data-y': 1000, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 1600, 'data-y': 2000, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 1600, 'data-y': 2000, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 1600, 'data-y': 2000, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 1560, 'data-y': 1800, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 3160, 'data-y': 1800, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 4760, 'data-y': 1800, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 6360, 'data-y': 1800, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 0, 'data-y': 0, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 3000, 'data-y': 1000, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
        ])

    def test_absolute_path(self):
        # Position slides along a path
        positions = [
            {'data-x': 'r0', 'data-y': 'r0', 'path': 'M 100 100 L 300 100 L 300 300',
             'is_path': True},
            {'is_path': True},
            {'is_path': True},
            {'is_path': True},
            {'is_path': True},
        ]

        positions = list(calculate_positions(positions))

        self.assertEqual(positions, [
            {'data-x': 0, 'data-y': 0, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 2000, 'data-y': 0, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 4000, 'data-y': 0, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 44.99999999999999, 'data-scale': 1},
            {'data-x': 4000, 'data-y': 2000, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 90.0, 'data-scale': 1},
            {'data-x': 4000, 'data-y': 4000, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 90.0, 'data-scale': 1},
        ])

    def test_relative_path(self):
        positions = [
            {'data-x': 'r0', 'data-y': 'r0'},
            {'data-x': 'r1600', 'data-y': 'r0'},
            {'data-x': 'r1600', 'data-y': 'r0', 'is_path': True,
             'path': 'm 100 100 l 200 0 l 0 200', },
            {'data-x': 'r0', 'data-y': 'r0', 'is_path': True},
            {'data-x': 'r0', 'data-y': 'r0', 'is_path': True},
            {'data-x': 'r1600', 'data-y': 'r0'},
            {'data-x': 'r0', 'data-y': 'r2400'},
        ]

        positions = list(calculate_positions(positions))

        self.assertEqual(positions, [
            {'data-x': 0, 'data-y': 0, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 1600, 'data-y': 0, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 3200, 'data-y': 0, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            # This point is exactly on a 90 degree angle. Therefore,
            # it's angle is calculated as 45 degrees, it being the
            # average.
            {'data-x': 5600, 'data-y': 0, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 44.99999999999999, 'data-scale': 1},
            {'data-x': 5600, 'data-y': 2400, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 90.0, 'data-scale': 1},
            {'data-x': 7200, 'data-y': 2400, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 90.0, 'data-scale': 1},
            {'data-x': 7200, 'data-y': 4800, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 90.0, 'data-scale': 1},
        ])

    def test_complex_path(self):
        positions = [
            {'data-x': 'r0', 'data-y': 'r0'},
            {'data-x': 'r1600', 'data-y': 'r0'},
            {'data-x': 'r1600', 'data-y': 'r0', 'path': 'm 100 100 l 200 0 l 0 200',
             'is_path': True},
            {'is_path': True},
            {'is_path': True},
            # Note that we don't change the rotation, so it stays at 90, here.
            {'data-x': '0', 'data-y': '0'},
            # No new x and y, previous was absolute: Stay still!
            {},
            {'data-x': 'r0', 'data-y': 'r0', 'path': 'm 100 100 l 200 0 l 0 200', 'is_path': True},
            {'is_path': True},
            {'is_path': True},
            {'data-x': '3000', 'data-y': '1000', 'data-rotate-z': '0'},
        ]

        positions = list(calculate_positions(positions))

        self.assertEqual(positions, [
            {'data-x': 0, 'data-y': 0, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 1600, 'data-y': 0, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 3200, 'data-y': 0, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 5600, 'data-y': 0, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 44.99999999999999, 'data-scale': 1},
            {'data-x': 5600, 'data-y': 2400, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 90.0, 'data-scale': 1},
            # Note that we don't change the rotation, so it stays at 90, here.
            {'data-x': 0, 'data-y': 0, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 90.0, 'data-scale': 1},
            # No settings, still same place and rotation.
            {'data-x': 0, 'data-y': 0, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 90.0, 'data-scale': 1},
            # We start a path, but x and y are r0, so no movement.
            # However, the rotation will come from the path, so it resets to 0.
            {'data-x': 0, 'data-y': 0, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
            {'data-x': 2400, 'data-y': 0, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 44.99999999999999, 'data-scale': 1},
            {'data-x': 2400, 'data-y': 2400, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 90.0, 'data-scale': 1},
            {'data-x': 3000, 'data-y': 1000, 'data-z': 0,
             'data-rotate-x': 0, 'data-rotate-y': 0,
             'data-rotate-z': 0, 'data-scale': 1},
        ])


class PositionTest(unittest.TestCase):

    def test_complete(self):
        tree = make_tree('positioning.rst')
        # Position the slides:
        position_slides(tree)

        # Get all slide position data:

        positions = []
        for step in tree.findall('step'):
            pos = {}
            for key in step.attrib:
                if key.startswith('data-'):
                    pos[key] = step.attrib[key]

            positions.append(pos)

        self.assertEqual(positions, [
            {'data-x': '0', 'data-y': '0', 'data-z': '0',
             'data-rotate-x': '0', 'data-rotate-y': '0',
             'data-rotate-z': '0', 'data-scale': '1'},
            {'data-x': '1600', 'data-y': '0', 'data-z': '0',
             'data-rotate-x': '0', 'data-rotate-y': '0',
             'data-rotate-z': '0', 'data-scale': '1'},
            # Because of the path, we now get an explicit rotation:
            {'data-x': '3200', 'data-y': '0', 'data-z': '0',
             'data-rotate-x': '0', 'data-rotate-y': '0',
             'data-rotate-z': '0', 'data-scale': '1'},
            {'data-x': '5600', 'data-y': '0', 'data-z': '0',
             'data-rotate-x': '0', 'data-rotate-y': '0',
             'data-rotate-z': '44.99999999999999', 'data-scale': '1'},
            {'data-x': '5600', 'data-y': '2400', 'data-z': '0',
             'data-rotate-x': '0', 'data-rotate-y': '0',
             'data-rotate-z': '90.0', 'data-scale': '1'},
            # Rotation carries over from last part of path.
            {'data-x': '0', 'data-y': '0', 'data-z': '0',
             'data-rotate-x': '0', 'data-rotate-y': '0',
             'data-rotate-z': '90.0', 'data-scale': '1'},
            # No position change
            {'data-x': '0', 'data-y': '0', 'data-z': '0',
             'data-rotate-x': '0', 'data-rotate-y': '0',
             'data-rotate-z': '90', 'data-scale': '1'},
            # No change at all.
            {'data-x': '0', 'data-y': '0', 'data-z': '0',
             'data-rotate-x': '0', 'data-rotate-y': '0',
             'data-rotate-z': '90', 'data-scale': '1'},
            # Path starts, rotation comes from path:
            {'data-x': '0', 'data-y': '0', 'data-z': '0',
             'data-rotate-x': '0', 'data-rotate-y': '0',
             'data-rotate-z': '0', 'data-scale': '1'},
            {'data-x': '2400', 'data-y': '0', 'data-z': '0',
             'data-rotate-x': '0', 'data-rotate-y': '0',
             'data-rotate-z': '44.99999999999999', 'data-scale': '1'},
            # Explicit rotate-x and z, automatic position including rotate-z from path.
            {'data-x': '2400', 'data-y': '2400', 'data-z': '1000',
             'data-rotate-x': '180', 'data-rotate-y': '0',
             'data-rotate-z': '90.0', 'data-scale': '1'},
            # Explicit x and y, all other carry over from last slide.
            {'data-x': '3000', 'data-y': '1000', 'data-z': '1000',
             'data-rotate-x': '180', 'data-rotate-y': '0',
             'data-rotate-z': '90.0', 'data-scale': '1'},
            # Positioning relative to other slide by id
            {'data-x': '4000', 'data-y': '500', 'data-z': '1000',
             'data-rotate-x': '180', 'data-rotate-y': '0',
             'data-rotate-z': '90.0', 'data-scale': '1'},
            # Positioning x relative to other slide by id, Explicit y
            {'data-x': '4800', 'data-y': '200', 'data-z': '1000',
             'data-rotate-x': '180', 'data-rotate-y': '0',
             'data-rotate-z': '90.0', 'data-scale': '1'},
        ])


if __name__ == '__main__':
    unittest.main()



import os
import re
import shutil
from lxml import etree, html
from pkg_resources import resource_string

from .parse import rst2xml, SlideMaker
from .position import position_slides
from .template import (Template, CSS_RESOURCE, JS_RESOURCE, JS_POSITION_HEADER,
                       JS_POSITION_BODY, OTHER_RESOURCE, DIRECTORY_RESOURCE)


class ResourceResolver(etree.Resolver):

    def resolve(self, url, pubid, context):
        if url.startswith('resource:'):
            prefix, filename = url.split(':', 1)
            return self.resolve_string(resource_string(__name__, filename), context)


def rst2html(filepath, template_info, auto_console=False, skip_help=False, skip_notes=False, mathjax=False, slide_numbers=False):
    # Read the infile
    with open(filepath, 'rb') as infile:
        rststring = infile.read()

    presentation_dir = os.path.split(filepath)[0]

    # First convert reST to XML
    xml, dependencies = rst2xml(rststring, filepath)
    tree = etree.fromstring(xml)

    # Fix up the resulting XML so it makes sense
    sm = SlideMaker(tree, skip_notes=skip_notes)
    tree = sm.walk()

    # Pick up CSS information from the tree:
    console_css = 'css/impressConsole.css'
    preview_css = None
    for attrib in tree.attrib:
        if attrib.startswith('css'):
            if '-' in attrib:
                dummy, media = attrib.split('-', 1)
            else:
                media = 'screen,projection'
            css_files = tree.attrib[attrib].split()
            for css_file in css_files:
                if media == 'console':
                    # The "console" media is used to style the presenter console
                    console_css = css_file
                    media = 'screen,projection'
                if media == 'preview':
                    # The "preview" media is used to style the presenter console previews.
                    preview_css = css_file
                    media = 'screen,projection'

                template_info.add_resource(
                    os.path.abspath(os.path.join(presentation_dir, css_file)),
                    CSS_RESOURCE,
                    target=css_file,
                    extra_info=media)
        if attrib.startswith('js'):
            if attrib == 'js-header':
                media = JS_POSITION_HEADER
            else:
                # Put javascript in body tag as default.
                media = JS_POSITION_BODY
            js_files = tree.attrib[attrib].split()
            for js_file in js_files:
                template_info.add_resource(
                    os.path.abspath(os.path.join(presentation_dir, js_file)),
                    JS_RESOURCE,
                    target=js_file,
                    extra_info=media)

    if sm.need_mathjax and mathjax:
        if mathjax.startswith('http'):
            template_info.add_resource(None, JS_RESOURCE,
                                       target=mathjax,
                                       extra_info=JS_POSITION_HEADER)
        else:
            # Local copy
            template_info.add_resource(mathjax, DIRECTORY_RESOURCE,
                                       target='mathjax')
            template_info.add_resource(None, JS_RESOURCE,
                                       target='mathjax/MathJax.js?config=TeX-MML-AM_CHTML',
                                       extra_info=JS_POSITION_HEADER)

    # Position all slides
    position_slides(tree)

    # Add the template info to the tree:
    tree.append(template_info.xml_node())

    # If the console-should open automatically, set an attribute on the document:
    if auto_console:
        tree.attrib['auto-console'] = 'True'

    # If the console-should open automatically, set an attribute on the document:
    if skip_help:
        tree.attrib['skip-help'] = 'True'

    # If the slide numbers should be displayed, set an attribute on the document:
    if slide_numbers:
        tree.attrib['slide-numbers'] = 'True'

    tree.attrib['console-css'] = console_css
    if preview_css:
        tree.attrib['preview-css'] = preview_css

    # We need to set up a resolver for resources, so we can include the
    # reST.xsl file if so desired.
    parser = etree.XMLParser()
    parser.resolvers.add(ResourceResolver())

    # Transform the tree to HTML
    xsl_tree = etree.fromstring(template_info.xsl, parser)
    transformer = etree.XSLT(xsl_tree)
    tree = transformer(tree)
    result = html.tostring(tree)

    return template_info.doctype + result, dependencies


def copy_resource(filename, sourcedir, targetdir):
    if filename[0] == '/' or ':' in filename:
        # Absolute path or URI: Do nothing
        return None  # No monitoring needed
    sourcepath = os.path.join(sourcedir, filename)
    targetpath = os.path.join(targetdir, filename)

    if (os.path.exists(targetpath) and
        os.path.getmtime(sourcepath) <= os.path.getmtime(targetpath)):
        # File has not changed since last copy, so skip.
        return sourcepath  # Monitor this file

    targetdir = os.path.split(targetpath)[0]
    if not os.path.exists(targetdir):
        os.makedirs(targetdir)

    shutil.copy2(sourcepath, targetpath)
    return sourcepath  # Monitor this file


def generate(args):
    """Generates the presentation and returns a list of files used"""

    source_files = {args.presentation}

    # Parse the template info
    template_info = Template(args.template)
    if args.css:
        presentation_dir = os.path.split(args.presentation)[0]
        target_path = os.path.relpath(args.css, presentation_dir)
        template_info.add_resource(args.css, CSS_RESOURCE, target=target_path, extra_info='all')
        source_files.add(args.css)
    if args.js:
        presentation_dir = os.path.split(args.presentation)[0]
        target_path = os.path.relpath(args.js, presentation_dir)
        template_info.add_resource(args.js, JS_RESOURCE, target=target_path, extra_info=JS_POSITION_BODY)
        source_files.add(args.js)

    # Make the resulting HTML
    htmldata, dependencies = rst2html(args.presentation, template_info,
                                      args.auto_console, args.skip_help,
                                      args.skip_notes, args.mathjax,
                                      args.slide_numbers)
    source_files.update(dependencies)

    # Write the HTML out
    if not os.path.exists(args.targetdir):
        os.makedirs(args.targetdir)
    with open(os.path.join(args.targetdir, 'index.html'), 'wb') as outfile:
        outfile.write(htmldata)

    # Copy supporting files
    source_files.update(template_info.copy_resources(args.targetdir))

    # Copy images from the source:
    sourcedir = os.path.split(os.path.abspath(args.presentation))[0]
    tree = html.fromstring(htmldata)
    for image in tree.iterdescendants('img'):
        filename = image.attrib['src']
        source_files.add(copy_resource(filename, sourcedir, args.targetdir))

    RE_CSS_URL = re.compile(br"""url\(['"]?(.*?)['"]?[\)\?\#]""")

    # Copy any files referenced by url() in the css-files:
    for resource in template_info.resources:
        if resource.resource_type != CSS_RESOURCE:
            continue
        # path in CSS is relative to CSS file; construct source/dest accordingly
        css_base = template_info.template_root if resource.is_in_template else sourcedir
        css_sourcedir = os.path.dirname(os.path.join(css_base, resource.filepath))
        css_targetdir = os.path.dirname(os.path.join(args.targetdir, resource.final_path()))
        uris = RE_CSS_URL.findall(template_info.read_data(resource))
        uris = [uri.decode() for uri in uris]
        if resource.is_in_template and template_info.builtin_template:
            for filename in uris:
                template_info.add_resource(filename, OTHER_RESOURCE, target=css_targetdir,
                                           is_in_template=True)
        else:
            for filename in uris:
                source_files.add(copy_resource(filename, css_sourcedir, css_targetdir))

    # All done!

    return {os.path.abspath(f) for f in source_files if f}
